/*
 *  m_sping.c: Implements timed server-server pinging
 *
 *  Copyright 2003 by W. Campbell
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1.Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  2.Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  3.The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  $Id$
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void ms_sping(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void mo_sping(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void ms_spong(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);

struct Message sping_msgtab = {
  "SPING", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, ms_sping, mo_sping}
};

struct Message spong_msgtab = {
  "SPONG", 0, 0, 5, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, ms_spong, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&sping_msgtab);
  mod_add_cmd(&spong_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&sping_msgtab);
  mod_del_cmd(&spong_msgtab);
}

const char *_version = "$Revision$";
#endif

static void ms_sping(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
  if (hunt_server(client_p, source_p, ":%s SPING %s %s %s %s",
                  1, parc, parv) == HUNTED_ISME)
  {
    sendto_one(source_p, ":%s SPONG %s %s %s %s", me.name, parv[0],
               parv[2], parv[3], parv[4]);
  }
}

static void ms_spong(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
  struct timeval nowtv;
  struct timeval prevtv;
  struct Client *target_p;
  double tdiff;

  if (hunt_server(client_p, source_p, ":%s SPONG %s %s %s %s",
                  1, parc, parv) == HUNTED_ISME)
  {
    /* Get the current time first, in order to prevent computational time
    ** from being counted against a server
    */
    gettimeofday(&nowtv, NULL);
    prevtv.tv_sec = strtol(parv[3], NULL, 10);
    prevtv.tv_usec = strtol(parv[4], NULL, 10);

    if (nowtv.tv_sec > prevtv.tv_sec)
    {
      nowtv.tv_usec += 1000000; /* Yes this is lame, manually do a subtraction
                                ** carry
                                */
      nowtv.tv_sec--;
    }

    /* Assume that tv_usec in nowtv is always greater than prevtv's */
    target_p = find_person(parv[2]);
    if (target_p == NULL)
      return;
    tdiff = (nowtv.tv_sec - prevtv.tv_sec) +
            ((nowtv.tv_usec - prevtv.tv_usec) / 1000000.0);
    sendto_one(target_p, ":%s NOTICE %s :SPONG from %s:  %1.3f seconds",
               me.name, target_p->name, source_p->name, tdiff);
  }
}

static void mo_sping(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[])
{
  struct timeval nowtv;
  char *argv[6];
  int argc;
  char secbuf[12];
  char usecbuf[8];

  gettimeofday(&nowtv, NULL);

  /* Syntax:  :sourceserver SPING destserver user tv_sec tv_usec */
  argv[0] = me.name;
  argv[1] = parv[1];
  argv[2] = parv[0];
  snprintf(secbuf, 12, "%ld", nowtv.tv_sec);
  argv[3] = secbuf;
  snprintf(usecbuf, 8, "%ld", nowtv.tv_usec);
  argv[4] = usecbuf;
  argc = 5;

  if (hunt_server(client_p, source_p, ":%s SPING %s %s %s %s",
                  1, argc, argv) == HUNTED_ISME)
  {
    sendto_one(source_p, ":%s NOTICE %s :Cannot SPING myself!", me.name,
               parv[0]);
  }
}
